"""
Upon importing loads configurations from the files that are provided as
commandline argument with `--config`.
"""
import os
import argparse
import warnings
from pathlib import Path
from conf.parsers import parse_from_yaml, parse_from_json, parse_from_ini
import conf


get = globals().get  # Capture the `get` method from `dict`.


def load(*names, override=True, raise_exception=False):
    """
    Read the given names and load their content into this configuration module.
    :param names: a varg that contains paths (str) to the conf files
    that need to be read or names of environment variables with paths.
    :param override: determines whether previously known configurations need to
    be overridden.
    :param raise_exception: Raise exception on parse failure
    :return: None.
    """
    for name in names:
        if not name:
            warnings.warn('an empty name is not allowed')
            return
        fname = os.environ.get(name, name)
        file_path = Path(fname)
        suffix = file_path.suffix or 'default'
        if not file_path.exists():
            warnings.warn('conf file "%s" not found' % fname)
            return
        parser = _supported_types.get(suffix.lower(), None)
        if not parser:
            warnings.warn('cannot parse files of type "%s"' % suffix)
            return
        with open(fname) as file:
            try:
                configurations = parser(file)
            except Exception as err:
                warnings.warn('failed to parse "%s". Reason: %s' %
                              (fname, err))
                if raise_exception:
                    raise
                else:
                    return
        for key in configurations:
            if override or not get(key):
                setattr(conf, key, configurations[key])
                globals()[key] = configurations[key]
                _content[key] = configurations[key]


def asdict() -> dict:
    """
    Get the loaded configuration as a dict.
    :return: the config as a dict.
    """
    return _content


_content = {}
_supported_types = {
    '.yml': parse_from_yaml,
    '.yaml': parse_from_yaml,
    '.json': parse_from_json,
    '.ini': parse_from_ini,
    'default': parse_from_ini
}
_help_msg = 'conf file(s) to load. Supported types are: %s' % \
            ', '.join(_supported_types)
_parser = argparse.ArgumentParser()
_parser.add_argument('--config', metavar='conf-file',
                     nargs='+', type=str, help=_help_msg)
_parsed_config = _parser.parse_known_args()[0]

if _parsed_config.config:
    load(*_parsed_config.config, override=True)



"""
This module contains the parsers that parse file streams to dicts.
"""
import configparser
import json
import yaml


def parse_from_yaml(file_stream):
    # Parse the given file stream of yaml format to a dict.
    loaded = yaml.load(file_stream, Loader=yaml.FullLoader)
    if loaded:
        return dict(loaded.items())
    return {}


def parse_from_json(file_stream):
    # Parse the given file stream of json format to a dict.
    return json.load(file_stream)


def parse_from_ini(file_stream):
    # Parse the given file stream of ini format to a dict.
    parser = configparser.ConfigParser()
    parser.read_file(file_stream)
    return {section: dict(parser[section].items())
            for section in parser.sections()}



"""
Generic configuration module.

|
| Start your application using:
|
| ``application.py --config CONFIGFILE.yml``
|
|
| You can also provide multiple configurations:
|
| ``application.py --config CONFIGFILE1.yml CONFIGFILE2.yml CONFIGFILE3.yml``
|
|
| When using multiple conf files, the latter files may override settings
| from the former files.
|
| In your application:
|
| ``import conf``
| ``my_setting = conf.get('my_setting', 'DEFAULT_SETTING')``
|
|
| If you don't need a default setting - make sure you always supply a conf
| file and that the setting exists:
|
| ``my_setting = conf.my_setting``
|
|
| Or simply:
|
| ``from conf import my_setting``
|
|
| If you use an ArgumentParser in your own application for other purposes, you
| must use the ``parse_known_args()`` method of the parser.
|
| Alternatively, you can load configuration files manually, using the load
| function. If you do this, make sure that the path is absolute. The syntax is:
|
| ``conf.load('complete/path/to/conf.yml')``
"""

from conf.reader import *



from setuptools import setup

with open('README.rst', 'r') as fh:
    long_description = fh.read()

setup(name='conf',
      version='0.3.1',
      author='Ramon Hagenaars',
      author_email='ramon.hagenaars@gmail.com',
      description='Facilitates a convenient use of configuration files',
      long_description=long_description,
      long_description_content_type='text/x-rst',
      url='https://github.com/ramonhagenaars/conf',
      packages=['conf'],
      install_requires=['PyYAML'],
      zip_safe=False,
      classifiers=(
          'Programming Language :: Python :: 3',
          'Operating System :: OS Independent'))
